@@ -1,25 +1,29 @@
-# Copyrights 2007-2017 by [Mark Overmeer].
+# Copyrights 2007-2018 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-use warnings;
-use strict;
+# This code is part of distribution Log-Report-Lexicon. Meta-POD processed
+# with OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Log::Report::Lexicon::Index;
 use vars '$VERSION';
-$VERSION = '1.08';
+$VERSION = '1.10';
 
 
+use warnings;
+use strict;
+
 use Log::Report       'log-report-lexicon';
 use Log::Report::Util  qw/parse_locale/;
 use File::Find         ();
 
 # The next two need extension when other lexicon formats are added
-sub _understand_file_format($) { $_[0] =~ qr/\.[mp]o$/i }
+sub _understand_file_format($) { $_[0] =~ qr/\.(?:gmo|mo|po)$/i }
 
 sub _find($$)
 {   my ($index, $name) = (shift, lc shift);
-    $index->{"$name.mo"} || $index->{"$name.po"};  # prefer mo
+    $index->{"$name.mo"} || $index->{"name.gmo"} || $index->{"$name.po"};  # prefer mo
 }
 
 # On windows, other locale names are used.  They will get translated
@@ -42,9 +46,11 @@ sub new($;@)
     bless {dir => $dir, @_}, $class;  # dir before first argument.
 }
 
+#-------------------
 
 sub directory() {shift->{dir}}
 
+#-------------------
 
 sub index() 
 {   my $self = shift;
@@ -99,9 +105,9 @@ sub find($$)
     $cs    = defined $cs    ? '.'.$cs    : '';
     $modif = defined $modif ? '@'.$modif : '';
 
-    (my $normcs = $cs) =~ s/[^a-z\d]//g;
+    (my $normcs = $cs) =~ s/[^a-z0-9]//g;
     if(length $normcs)
-    {   $normcs = "iso$normcs" if $normcs !~ /\D/;
+    {   $normcs = "iso$normcs" if $normcs !~ /[^0-9-]/;
         $normcs = '.'.$normcs;
     }
 
