@@ -1,43 +1,52 @@
-# Copyrights 2007-2017 by [Mark Overmeer].
+# Copyrights 2007-2018 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-use warnings;
-use strict;
+# This code is part of distribution Log-Report-Lexicon. Meta-POD processed
+# with OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Log::Report::Lexicon::MOTcompact;
 use vars '$VERSION';
-$VERSION = '1.08';
+$VERSION = '1.10';
 
 use base 'Log::Report::Lexicon::Table';
 
-use Log::Report        'log-report-lexicon';
-use Fcntl              'SEEK_SET';
-use Encode             'decode';
+use warnings;
+use strict;
+
+use Log::Report  'log-report-lexicon';
+use Fcntl        qw(SEEK_SET);
+use Encode       qw(find_encoding);
 
 use constant MAGIC_NUMBER => 0x95_04_12_DE;
 
 
 sub read($@)
 {   my ($class, $fn, %args) = @_;
-    my $take_all = exists $args{take_all} ? $args{take_all} : 1;
 
-    my $charset  = $args{charset}
-        or error __x"charset parameter required for {fn}", fn => $fn;
+    my $charset  = $args{charset};
+    $charset    = $1
+        if !$charset && $fn =~ m!\.([\w-]+)(?:\@[^/\\]+)?\.g?mo$!i;
+
+    my $enc;
+    if(defined $charset)
+    {   $enc = find_encoding($charset)
+            or error __x"unsupported explicit charset {charset} for {fn}"
+                , charset => $charset, fn => $fn;
+    }
 
     my (%index, %locs);
     my %self     =
      +( index    => \%index   # fully prepared ::PO objects
       , locs     => \%locs    # know where to find it
       , filename => $fn
-      , charset  => $charset
       );
     my $self    = bless \%self, $class;
 
     my $fh;
     open $fh, "<:raw", $fn
-        or fault __x"cannot read in {cs} from file {fn}"
-             , cs => $charset, fn => $fn;
+        or fault __x"cannot read mo from file {fn}", fn => $fn;
 
     # The magic number will tell us the byte-order
     # See http://www.gnu.org/software/gettext/manual/html_node/MO-Files.html
@@ -87,7 +96,8 @@ sub read($@)
 
     # We need the originals as index to the translations (unless there
     # is a HASH build-in... which is not defined)
-    # The strings are strictly ordered, the spec tells me.
+    # The strings are strictly ordered, the spec tells me, to allow binary
+    # search.  Better swiftly process the whole block into a hash.
     my ($orig_start, $orig_end) = ($origs[1], $origs[-1]+$origs[-2]);
 
     seek $fh, $orig_start, SEEK_SET
@@ -105,71 +115,58 @@ sub read($@)
         or fault __x"cannot seek to {loc} in {fn} for transl strings"
           , loc => $trans_start, fn => $fn;
 
-    if($take_all)
-    {   my $trans_block_size = $trans_end - $trans_start;
-        CORE::read $fh, $trans_block, $trans_block_size
-            or fault __x"cannot read translations from {fn}, need {size} at {loc}"
-               , fn => $fn, loc => $trans_start, size => $trans_block_size;
-    }
+    my $trans_block_size = $trans_end - $trans_start;
+    CORE::read $fh, $trans_block, $trans_block_size
+        or fault __x"cannot read translations from {fn}, need {size} at {loc}"
+          , fn => $fn, loc => $trans_start, size => $trans_block_size;
 
     while(@origs)
     {   my ($id_len, $id_loc) = (shift @origs, shift @origs);
         my $msgid_b   = substr $orig_block, $id_loc-$orig_start, $id_len;
         my $msgctxt_b = $msgid_b =~ s/(.*)\x04// ? $1 : '';
 
-        my $msgid     = decode $charset, $msgid_b;
-        my $msgctxt   = decode $charset, $msgctxt_b;
-
         my ($trans_len, $trans_loc) = (shift @trans, shift @trans);
-        if($take_all)
-        {   my $msgstr_b = substr $trans_block, $trans_loc - $trans_start, $trans_len;
-            my @msgstr   = map decode($charset, $_), split /\0x00/, $msgstr_b;
-            $index{"$msgid#$msgctxt"} = @msgstr > 1 ? \@msgstr : $msgstr[0];
-        }
-        else
-        {   # this may save memory...
-            $locs{"$msgid#$msgctxt"}  = [$trans_loc, $trans_len];
+        my $msgstr_b = substr $trans_block, $trans_loc - $trans_start, $trans_len;
+
+        unless(defined $charset)
+        {    $msgid_b eq ''
+                 or error __x"the header is not the first entry, needed for charset in {fn}", fn => $fn;
+
+             $charset = $msgstr_b =~ m/^content-type:.*?charset=["']?([\w-]+)/mi
+                ? $1 : error __x"cannot detect charset in {fn}", fn => $fn;
+             trace "auto-detected charset $charset for $fn";
+
+             $enc = find_encoding($charset)
+                  or error __x"unsupported charset {charset} in {fn}"
+                      , charset => $charset, fn => $fn;
         }
+
+        my $msgid   = $enc->decode($msgid_b);
+        my $msgctxt = $enc->decode($msgctxt_b);
+        my @msgstr  = map $enc->decode($_), split /\0x00/, $msgstr_b;
+        $index{"$msgid#$msgctxt"} = @msgstr > 1 ? \@msgstr : $msgstr[0];
     }
 
-    if($take_all)
-    {   close $fh
-           or failure __x"failed reading from file {fn}", fn => $fn;
-    }
-    else
-    {   $self->{fh} = $fh;
-    }
+    close $fh
+         or failure __x"failed reading from file {fn}", fn => $fn;
 
+    $self->{origcharset} = $charset;
     $self->setupPluralAlgorithm;
     $self;
 }
 
 #---------
 
-sub index()     {shift->{index}}
-sub filename()  {shift->{filename}}
+sub index() {shift->{index}}
+sub filename() {shift->{filename}}
+sub originalCharset() {shift->{origcharset}}
 
+#---------------
 
 sub msgid($;$)
 {   my ($self, $msgid, $msgctxt) = @_;
     my $tag = $msgid.'#'.($msgctxt//'');
-    my $po  = $self->{index}{$tag};
-    return $po if $po;
-
-    my $l   = delete $self->{locs}{$tag} or return ();
-
-    my $fh  = $self->{fh};
-    seek $fh, $l->[0], SEEK_SET
-        or fault __x"cannot seek to {loc} late in {fn} for transl strings"
-          , loc => $l->[0], fn => $self->filename;
-
-    my $block;
-    CORE::read $fh, $block, $l->[1]
-      or fault __x"cannot read late translation from {fn}, need {size} at {loc}"
-          , fn => $self->filename, loc => $l->[0], size => $l->[1];
-
-    my @msgstr = split /\0x00/, $block;
-    $self->{index}{$tag} = @msgstr > 1 ? \@msgstr : $msgstr[0]; 
+    $self->{index}{$tag};
 }
 
 
