@@ -1,15 +1,19 @@
-# Copyrights 2007-2017 by [Mark Overmeer].
+# Copyrights 2007-2018 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-use warnings;
-use strict;
+# This code is part of distribution Log-Report-Lexicon. Meta-POD processed
+# with OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Log::Report::Lexicon::PO;
 use vars '$VERSION';
-$VERSION = '1.07';
+$VERSION = '1.10';
 
 
+use warnings;
+use strict;
+
 use Log::Report 'log-report-lexicon';
 
 # steal from cheaper module, we have no ::Util for this (yet)
@@ -212,7 +216,7 @@ sub addFlags($)
 sub fromText($$)
 {   my $class = shift;
     my @lines = split /[\r\n]+/, shift;
-    my $where = shift || ' unkown location';
+    my $where = shift || ' unknown location';
 
     my $self  = bless {}, $class;
 
