@@ -1,22 +1,27 @@
-# Copyrights 2007-2017 by [Mark Overmeer].
+# Copyrights 2007-2018 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-use warnings;
-use strict;
+# This code is part of distribution Log-Report-Lexicon. Meta-POD processed
+# with OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Log::Report::Lexicon::POT;
 use vars '$VERSION';
-$VERSION = '1.08';
+$VERSION = '1.10';
 
 use base 'Log::Report::Lexicon::Table';
 
+use warnings;
+use strict;
+
 use Log::Report 'log-report-lexicon';
 use Log::Report::Lexicon::PO  ();
 
 use POSIX        qw/strftime/;
 use List::Util   qw/sum/;
 use Scalar::Util qw/blessed/;
+use Encode       qw/decode/;
 
 use constant     MSGID_HEADER => '';
 
@@ -25,10 +30,8 @@ sub init($)
 {   my ($self, $args) = @_;
 
     $self->{LRLP_fn}      = $args->{filename};
-    $self->{LRLP_index}   = $args->{index} || {};
-    $self->{LRLP_charset} = $args->{charset}
-        or error __x"charset parameter is required for {fn}"
-             , fn => ($args->{filename} || __"unnamed file");
+    $self->{LRLP_index}   = $args->{index}   || {};
+    $self->{LRLP_charset} = $args->{charset} || 'UTF-8';
 
     my $version    = $args->{version};
     my $domain     = $args->{textdomain}
@@ -55,15 +58,22 @@ sub init($)
 
 sub read($@)
 {   my ($class, $fn, %args) = @_;
-
     my $self    = bless {LRLP_index => {}}, $class;
 
-    my $charset = $self->{LRLP_charset} = $args{charset}
-        or error __x"charset parameter is required for {fn}", fn => $fn;
+    my $charset = $args{charset};
+    $charset    = $1
+        if !$charset && $fn =~ m!\.([\w-]+)(?:\@[^/\\]+)?\.po$!i;
 
-    open my $fh, "<:encoding($charset):crlf", $fn
-        or fault __x"cannot read in {cs} from file {fn}"
-             , cs => $charset, fn => $fn;
+    my $fh;
+    if(defined $charset)
+    {   open $fh, "<:encoding($charset):crlf", $fn
+            or fault __x"cannot read in {cs} from file {fn}"
+                 , cs => $charset, fn => $fn;
+    }
+    else
+    {   open $fh, '<:raw:crlf', $fn
+            or fault __x"cannot read from file {fn} (unknown charset)", fn=>$fn;
+    }
 
     local $/   = "\n\n";
     my $linenr = 1;  # $/ frustrates $fh->input_line_number
@@ -77,14 +87,27 @@ sub read($@)
         $block   =~ s/\s+\z//s;
         length $block or last;
 
+        unless($charset)
+        {   $charset = $block =~ m/\"content-type:.*?charset=["']?([\w-]+)/mi
+              ? $1 : error __x"cannot detect charset in {fn}", fn => $fn;
+            trace "auto-detected charset $charset for $fn";
+            $fh->binmode(":encoding($charset):crlf");
+
+            $block = decode $charset, $block
+               or error __x"unsupported charset {charset} in {fn}"
+                    , charset => $charset, fn => $fn;
+        }
+
         my $po = Log::Report::Lexicon::PO->fromText($block, $location);
         $self->add($po) if $po;
     }
 
     close $fh
-        or failure __x"failed reading from file {fn}", fn => $fn;
+        or fault __x"failed reading from file {fn}", fn => $fn;
+
+    $self->{LRLP_fn}      = $fn;
+    $self->{LRLP_charset} = $charset;
 
-    $self->{LRLP_fn} = $fn;
     $self->setupPluralAlgorithm;
     $self;
 }
