@@ -1,15 +1,19 @@
-# Copyrights 2007-2017 by [Mark Overmeer].
+# Copyrights 2007-2018 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-use warnings;
-use strict;
+# This code is part of distribution Log-Report-Lexicon. Meta-POD processed
+# with OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Log::Report::Lexicon::Table;
 use vars '$VERSION';
-$VERSION = '1.08';
+$VERSION = '1.10';
 
 
+use warnings;
+use strict;
+
 use Log::Report 'log-report-lexicon';
 
 use POSIX       qw/strftime/;
@@ -37,15 +41,16 @@ sub translations(;$) {panic "not implemented"}
 
 sub pluralIndex($)
 {   my ($self, $count) = @_;
-    my $algo = $self->{algo} or panic;
+    my $algo = $self->{algo}
+        or error __x"there is no Plural-Forms field in the header, but needed";
+
     $algo->($count);
 }
 
 
 sub setupPluralAlgorithm()
 {   my $self  = shift;
-    my $forms = $self->header('Plural-Forms')
-        or error __x"there is no Plural-Forms field in the header";
+    my $forms = $self->header('Plural-Forms') or return;
 
     my $alg   = $forms =~ m/plural\=([n%!=><\s\d|&?:()]+)/ ? $1 : "n!=1";
     $alg =~ s/\bn\b/(\$_[0])/g;
